  /******************************************************************
   *                                                                *
   *  Project: 18F14K22 2-Pin 8-Bit LCD Backpack Test               *
   *   Source: 18F14K22_2-Pin_8-Bit_LCD_Backpack_Test.c             *
   *   Author: Mike McLaren, K8LH                                   *
   *  (C)2013: Micro Application Consultants                        *
   *  Revised: 14-Jan-2013                                          *
   *                                                                *
   *  18F14K22 (16-MHz) experiment driving the K8LH 2-Pin 8-Bit     *
   *  LCD Backpack.                                                 *
   *                                                                *
   *                                                                *
   *      IDE: MPLAB 8.84 (tabs = 4)                                *
   *     Lang: Sourceboost BoostC v7.05, Lite/Free version          *
   *                                                                *
   ******************************************************************/

   #include <system.h>

   #pragma DATA _CONFIG1H, _FOSC_IRC_1H & _PLLEN_OFF_1H & _FCMEN_OFF_1H & _IESO_OFF_1H 
   #pragma DATA _CONFIG2L, _PWRTEN_ON_2L & _BOREN_SBORDIS_2L & _BORV_22_2L
   #pragma DATA _CONFIG2H, _WDTEN_OFF_2H & _WDTPS_256_2H
   #pragma DATA _CONFIG3H, _HFOFST_OFF_3H & _MCLRE_ON_3H
   #pragma DATA _CONFIG4L, _STVREN_ON_4L & _LVP_OFF_4L & _XINST_OFF_4L & _DEBUG_OFF_4L

  /*
   *  _FOSC_IRC_1H       Internal RC oscillator
   *  _PLLEN_OFF_1H      PLL is under software control
   *  _FCMEN_OFF_1H      Fail-Safe Clock Monitor disabled
   *  _IESO_OFF_1H       Oscillator Switchover mode disabled
   *  _PWRTEN_ON_2L      PWRT enabled (default)
   *  _BOREN_SBORDIS_2L  Brown-out Reset enabled in hardware
   *  _BORV_22_2L        VBOR set to 2.2 V nominal
   *  _WDTEN_OFF_2H      WDT controlled via WDTCON SWDTEN bit
   *  _WDTPS_256_2H      WDT prescaler 1:256
   *  _HFOFST_OFF_3H     Hold system clock until HFINTOSC stable
   *  _MCLRE_OFF_3H      RA3 input enabled, MCLR disabled
   *  _STVREN_ON_4L      reset on Stack full or underflow
   *  _LVP_OFF_4L        Single-Supply ICSP disabled
   *  _XINST_OFF_4L      extended instruction set disabled
   *  _DEBUG_OFF_4L      debugger off, RA0 + RA1 available
   */
   

   #pragma CLOCK_FREQ 16000000  // 16-MHz INTOSC

  /******************************************************************
   *  function prototypes                                           *
   ******************************************************************/
  /******************************************************************
   *  variables                                                     *
   ******************************************************************/

   char flags = 0;              // assorted flag bits

  /******************************************************************
   *  defines/constants                                             *
   ******************************************************************/

   #define line1 128+0          // lcd ddram address for line 1
   #define line2 128+64         // lcd ddram address for line 2

   #define clk  lata.1          // 595 'sck' & 'ser' + lcd 'rs'
   #define ena  lata.2          // 595 'rck' + lcd 'e' lines
   #define rs   flags.0         // lcd 'rs' flag bit

  /******************************************************************
   *  low level drivers & functions                                 *
   ******************************************************************/

  /*                                                                *
   *  K8LH 2-Pin 8-Bit 74HC595 LCD low level driver (isochronous)   *
   *                                                                */
   void PutLCD(char work)       // write byte to 74HC595 & LCD
   { char bitctr = 8;           //
     do                         // start of bit clock cycle
     { if(!work.7)              // if b7 is a '0' bit
         clk = 0;               // start clock cycle for a '0'
       delay_us(9);             // charge or drain cap to 3t
       clk = 0;                 // start clock cycle for a '1'
       clk = 1;                 // clock out the '0' or '1'
       work <<= 1;              // shift next bit into b7
     } while(--bitctr);         // until all 8 bits clocked out
     if(rs == 0) clk = 0;       // make clk pin = rs flag
     ena = 1; ena = 0;          // latch 595, pulse lcd 'e' pin
     clk = 1;                   // always leave clk pin high
   }

   void PutCMD(char pdata)      // lcd command (RS=0)
   { rs = 0; PutLCD(pdata);     // rs = 0, send command
   }                            //

   void PutDAT(char pdata)      // lcd data (RS=1)
   { rs = 1; PutLCD(pdata);     // rs = 1, send data
   }                            //

   void PutDAT(char *pdata)     // lcd data (RS=1) strings
   { while(*pdata)              //
       PutDAT(*pdata++);        //
   }                            //

  /*                                                                *
   *  initialize lcd (8-bit interface mode)                         *
   *                                                                */
   void lcdinit()               //
   { delay_ms(100);             //
     PutCMD(0x38);              // 8-bit, 2-lines, 5x7 font
     PutCMD(0x0C);              // display on, currsor & blink off
     PutCMD(0x06);              // cursor inc, shift off
     PutCMD(0x01);              // clear display
     delay_ms(2);               // required delay
   }                            //

  /******************************************************************
   *  main init                                                     *
   ******************************************************************/

   void main()                  //
   { 
     ansel = 0;                 //
     anselh = 0;                //
     osccon = 0b01110010;       // INTOSC = 16-MHz
     trisa = 0b00001000;        // RA3 input, others outputs
     trisb = 0b00000000;        // portb all outputs
     trisc = 0b00000000;        // portc all outputs
     porta = 1<<RA2;            // RA2 (clk) pin must be high
     portc = 0b00000000;        // 

     lcdinit();                 // init LCD in 8-bit interface mode
     PutDAT("DVM Project");     // 

  /******************************************************************
   *  main loop                                                     *
   ******************************************************************/

     while(1)                   //
     { 
     }                          //
   }                            //
